/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * FILENAME
 *      azn_eas_demo.cpp
 *
 * DESCRIPTION
 *      Demonstration Extrernal Authorization Service.
 */

#include <azn_eas_demo.h>
#include <aznutils.h>
#include <azn_svc_protos.h>

#define AZN_C_INDIFFERENT (-1)
#define AZN_EAS_DEMO_VER  "Policy Director Demonstration External Authorization Server v1.0"


/* Make sure this file can be compiled using either the C or C++ compiler */
#ifdef __cplusplus
extern "C" {
#endif


/**************************************************************************
 * Interface function definitions.
 **************************************************************************/

/*
 * FUNCTION NAME
 *      azn_svc_initialize
 *
 * DESCRIPTION
 *      init the ext_attr entitlements service
 *
 * ARGUMENTS
 *      [in] argc       The count of arguments to the service.
 *
 *      [in] argv       The array of argument strings.
 *
 *      [in] svcInit    List of initialization attributes for the service.
 *
 *      [out] svcInfo   attr list ptr for attributes returned by the
 *                      service.
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 */

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_initialize(
  const int               argc,   	/* in */
  const char              **argv, 	/* in */
  const azn_attrlist_h_t  svcInit,	/* in */
  azn_attrlist_h_t  	  *svcInfo	/* out */
)
{
    azn_status_t    st;
    azn_boolean_t   freeAttrlist = FALSE;

    /* svcInfo must not be NULL */
    if (svcInfo == NULL) {
        return (azn_util_errcode(AZN_S_FAILURE,
                                 AZN_EAS_DEMO_INVALID_SVCINFO_HDL));
    }

    /* ensure argc is valid */
    if (argc < 0 || (argc == 0 && argv != NULL)) {
        return (azn_util_errcode(AZN_S_FAILURE,
                                 AZN_EAS_DEMO_INVALID_ARG_COUNT));
    }

    if (argc > 0 && argv == NULL) {
        return (azn_util_errcode(AZN_S_FAILURE,
                                 AZN_EAS_DEMO_INVALID_ARG_ARRAY));
    }

    /* return the service version to the dispatcher */
    if (*svcInfo == AZN_C_INVALID_HANDLE) {
        azn_attrlist_create(svcInfo);
        freeAttrlist = TRUE;
    }

    st = azn_attrlist_add_entry(*svcInfo, azn_svc_version,
                                AZN_EAS_DEMO_VER);

    if (st != AZN_S_COMPLETE) {
        if (freeAttrlist) {
            azn_attrlist_delete(svcInfo);
        }
        return (st);
    }

    return (azn_util_errcode(AZN_S_COMPLETE, 0));
}


/*
 * FUNCTION NAME
 *      azn_svc_shutdown
 *
 * DESCRIPTION
 *      shutdown the ext_attr entitlements service.
 *      The initialization parameters are passed in
 *      again on shutdown but are ignored. No version info
 *      is returned.
 *
 * ARGUMENTS
 *      [in] argc       The count of arguments to the service.
 *
 *      [in] argv       The array of argument strings.
 *
 *      [in] svcInit    List of initialization attributes for the service.
 *
 *      [out] svcInfo   attr list ptr for attributes returned by the
 *                      service.
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 */

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_shutdown(
  const int               argc,       /* in */
  const char              **argv,     /* in */
  const azn_attrlist_h_t  svcInit,    /* in */
  azn_attrlist_h_t  	  *svcInfo    /* out */
)
{
    return (azn_util_errcode(AZN_S_COMPLETE, 0));
}


/*
 * FUNCTION NAME
 *      azn_svc_decision_access_allowed_ext
 *
 * DESCRIPTION
 *  Performs an extended access decision request and returns a
 *  permission code and permission information to the caller.
 *
 * ARGUMENTS
 *      [in]  creds               The credentials of the caller.
 *      [in]  protected_resource  The target object.
 *      [in]  operation           The operation to be performed.
 *      [in]  app_context         Attribute list containing string
 *                                attributes to pass to service.
 *      [out] permission          The result of the decision.
 *      [out] permission_info     An attribute list containing
 *                                information concerning the access
 *                                decision.
 *                                The permission_info returned may
 *                                be AZN_C_INVALID_HANDLE if there
 *                                was no information to return.
 * RETURN VALUE
 *      AZN_S_COMPLETE and a valid decision in 'permission' on success.
 *      An error code on failure and AZN_C_NOT_PERMITTED in 'permission'.
 *      permission_info will be set if addition information need to be
 *      returned.
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_decision_access_allowed_ext(
    const azn_creds_h_t      creds,              /* in */
    const azn_string_t       protected_resource, /* in */
    const azn_string_t       operation,          /* in */
    const azn_attrlist_h_t   app_context,        /* in */
    int                      *permission,        /* out */
    azn_attrlist_h_t         *permission_info    /* out */
    )
{
    azn_status_t    st = AZN_S_COMPLETE;
    azn_boolean_t   freeAttrlist = FALSE;


    /*
     * Set default permission value returned if possible.
     * Ensure permission value is AZN_C_NOT_PERMITTED if
     * any errors occur processing parameters.
     */
    if (permission == NULL)
        return (azn_util_errcode(AZN_S_INVALID_PERMISSION_REF, 0));

    *permission = AZN_C_INDIFFERENT;

    /*
     * Check the parameters.
     */
    if (protected_resource == NULL)
        return (azn_util_errcode(AZN_S_INVALID_PROTECTED_RESOURCE, 0));

    if (operation == NULL)
        return (azn_util_errcode(AZN_S_INVALID_OPERATION, 0));

    if (creds == AZN_C_INVALID_HANDLE)
        return (azn_util_errcode(AZN_S_INVALID_CREDS_HDL, 0));


    /*
     * Return the service version as a permission_info
     * attribute if the permission_info ptr is !NULL.
     */
    if (permission_info != NULL) {
    if (!azn_util_handle_is_valid(*permission_info)) {
        *permission_info = AZN_C_INVALID_HANDLE;
    }

    if (*permission_info == AZN_C_INVALID_HANDLE) {
        st = azn_attrlist_create(permission_info);
        if (st != AZN_S_COMPLETE)
        return (st);
        freeAttrlist = TRUE;
    }

    st = azn_attrlist_add_entry(*permission_info, azn_svc_version,
                    AZN_EAS_DEMO_VER);

    if (st != AZN_S_COMPLETE) {
        if (freeAttrlist)
        azn_attrlist_delete(permission_info);
        return (st);
    }
    }

    *permission = AZN_C_PERMITTED;

    return (azn_util_errcode(AZN_S_COMPLETE, 0));
}


/* Make sure this file can be compiled using either the C or C++ compiler */
#ifdef __cplusplus
}
#endif

